/*
 * Author: Konrad Rzeszutek <conradus@mindless.com>
 * Date:   Oct 19th, 1999
 * File:   Editor3D/Matrix.java
 *
 * Notes:
 *
 *  Manipulates matrix representation and also multiplies diffrent matrix-es.
 *
 */
 package Editor3D;
 
public class Matrix {

    /*
     * Initializes the Matrix.
     *
     * require.
     *  rows >= 1 && columns >= 1
     *
     */
    public Matrix (int rows, int columns) {
    
	this.rows = rows;
	this.columns = columns;
	
	m = new double[rows][];
	temp = new double[rows][];
	
	// Create the columns.
    	for (int i=0; i < rows; i++) {
	    m[i] = new double[columns];
	    temp[i] = new double[columns];
	}
		
	// Make it an identity matrix. 
	
	for (int i=0; i < rows; i++) 
	    for (int j=0; j < columns; j++) 
	    if (i==j)
		m[i][j] = 1.0;
	    else
		m[i][j] = 0.0;
    }
    /*
     * Initializes the Matrix.
     * This will create a 4x4 identity matrix.
     */
    public Matrix () {
    
	this(4,4);
    }
    
    /*
     * Multiply m1 matrix against m2 matrix and put the result in 'this'
     *
     * require:
     *   m1.rows == m2.rows && m2.columns == m1.columns.
     */
    public void multiply(Matrix m1, Matrix m2) {
	
	for (int i = 0 ; i < rows; i++)
	    for (int j = 0; j < columns; j++) {
		temp[i][j] = 0.0; // Clear it
		for (int k = 0; k < columns; k++)
		    temp[i][j] += m1.m[i][k] * m2.m[k][j];
	    }
	// copy.
	for (int i = 0; i < rows; i++)
	    for (int j = 0 ; j < columns; j++)
		m[i][j] = temp[i][j];
    
    }	
    
    /*
     * Returns a string representation of the matrix.
     *
     * return:
     *   String.
     */
    public String toString() {
	
	String str = "\n";
	    
	for (int i = 0; i < rows; i++) {	    
	    for (int j = 0; j < columns; j++)
		str += "\t"+m[i][j];
	    str += "\n";
	    }
	    
	return str;
    }
    // Public variables 
    public int rows;     
    public int columns;
    
    public double m[][];   // The Matrix.
	
    private double temp[][]; // Placeholder.
    
}			