/*
 * Author: Konrad Rzeszutek <conradus@mindless.com>
 * Date:   Oct 19th 1999
 * File:   Editor3D/ViewPort.java
 *
 * Notes:
 *
 *   This object deals with the WC (World Co-ordinate) system. This system
 *  is used by the user for manipulating objects in his/her own system.
 *
 */
package Editor3D;
public class ViewPort {

    // Constants used for addressing u and v.
    public final static int MIN = 0;
    public final static int MAX = 1;


    /*
     * Initalize the ViewPort.
     *
     * require:
     *  (uv.length == 4) 
     */
    public ViewPort (double[] uv) {
    
	u = new double[2];
	v = new double[2];
	temp = new double[4];	
	vecVRP =  new VectorObject();
	vecVPN =  new VectorObject();
	vecVUP =  new VectorObject();
	
	vecCOP = new VectorObject();
	
	// Temp is represnented as: P^T=[Umin Vmin Umax Vmax]
	temp[0] = u[0] = uv[0];
	temp[1] = u[1] = uv[1];
	temp[2] = v[0] = uv[2];
	temp[3] = v[1] = uv[3];
    }
    /*
     * Initalize the ViewPort.
     *
     */
    public ViewPort() {
    
        u = new double[2];
	v = new double[2];
	temp = new double[4];	

	vecVRP = new VectorObject();
	vecVPN = new VectorObject();
	vecVUP = new VectorObject();
	
	vecCOP = new VectorObject();
	temp[0] = u[0] = 0;
	temp[1] = u[1] = 0;
	temp[2] = v[0] = 0;
	temp[3] = v[1] = 0;
    }
    
    
    /*
     * Set the UV for the ViewPort
     *
     * require:
     *  uv.lenght == 4.
     *
     */
    public void setUV(double[] uv) {
    
    	temp[0] = u[0] = uv[0];
	temp[1] = u[1] = uv[1];
	temp[2] = v[0] = uv[2];
	temp[3] = v[1] = uv[3];
    }
    
    /*
     * Set the new VRP (View Reference Point)
     *
     * require:
     *   vecVRP != null
     */
    public void setVRP(VectorObject vecVRP) {
        
	this.vecVRP = vecVRP;
    }
    /*
     * Set the new VPN (View Point Normal)
     *
     * require:
     *  vecVPN != null
     */
    public void setVPN(VectorObject vecVPN) {
    
	this.vecVPN = vecVPN;
    }
    
    /*
     * Set the new VUP (View ?)
     *
     * require:
     *  vecVUP != null
     */
    public void setVUP(VectorObject vecVUP) {
    
	this.vecVUP = vecVUP;
    }
    
    /*
     * Set the new COP (Central Of Perspecitive) also known as PRP
     *
     * require:
     *   vecCOP != null
     */
    public void setCOP (VectorObject vecCOP) {
    
	this.vecCOP = vecCOP;

    }
          
    /*
     * Reset the values of the ViewPort to the original ones 
     * supplied by the top program when this object was created.
     *
     */
    public void reset() {
    
	u[0] = temp[0];
	u[1] = temp[1];
	v[0] = temp[0];
	v[1] = temp[1];

	vecVRP.reset();
	vecVPN.reset();
	vecVUP.reset();
	vecCOP.reset();
	
    }
    /*
     * Returns a nice string representation of the ViewPorts data.
     *
     * return:
     *  String.
     */
    public String toString() {
    
	return "\n Umin = "+u[0]+
	       "\n Umax = "+u[1]+
	       "\n Vmin = "+v[0]+
	       "\n Vmax = "+v[1];
	       
    }
    
    // Public available variables.
    public double[] u;
    public double[] v;
    
    // Coordinates used for viewing from a 3D perspective.
    public VectorObject vecVRP;

    public VectorObject vecVPN;

    public VectorObject vecVUP;

    public VectorObject vecCOP;

    

    private double[] temp;
}